unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, EXIF_Intf, EXIF_Defs, ComCtrls, StdCtrls, ExtCtrls, ExtDlgs;

type
  TForm1 = class(TForm)
    Button1: TButton;
    opd: TOpenPictureDialog;
    List: TListView;
    Edit1: TEdit;
    tc: TTabControl;
    Image: TImage;
    ImagePanel: TPanel;
    Label1: TLabel;
    Memo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure cmOpen(Sender: TObject);
    procedure ListDblClick(Sender: TObject);
    procedure tcChange(Sender: TObject);
  private
    EXIF: TEXIF;
  public
    procedure DisplayIFD0;
    procedure DisplayIFD1;
    procedure DisplayEXIFSubIFD;
    procedure DisplayEXIFInteropIFD;
    procedure DisplayEXIFGPSIFD;
    procedure DisplayMakerNote;
    procedure DisplayThumbnail;
    procedure DisplayComment;
    procedure DisplayImage;
  end;

var
  Form1: TForm1;

implementation

uses JPEG;

{$R *.dfm}

procedure RotateBitmap(Bmp: TBitmap; Rads: Single; AdjustSize: Boolean; BkColor: TColor = clNone);
var
  C: Single;
  S: Single;
  Tmp: TBitmap;
  OffsetX: Single;
  OffsetY: Single;
  Points: array[0..2] of TPoint;
begin
  C:=Cos(Rads);
  S:=Sin(Rads);
  Tmp:=TBitmap.Create;
  try
    Tmp.TransparentColor:=Bmp.TransparentColor;
    Tmp.TransparentMode:=Bmp.TransparentMode;
    Tmp.Transparent:=Bmp.Transparent;
    Tmp.Canvas.Brush.Color:=BkColor;
    if AdjustSize then begin
      Tmp.Width:=Round(Bmp.Width * Abs(C) + Bmp.Height * Abs(S));
      Tmp.Height:=Round(Bmp.Width * Abs(S) + Bmp.Height * Abs(C));
      OffsetX:=(Tmp.Width - Bmp.Width * C + Bmp.Height * S) / 2;
      OffsetY:=(Tmp.Height - Bmp.Width * S - Bmp.Height * C) / 2;
    end else begin
      Tmp.Width:=Bmp.Width;
      Tmp.Height:=Bmp.Height;
      OffsetX:=(Bmp.Width - Bmp.Width * C + Bmp.Height * S) / 2;
      OffsetY:=(Bmp.Height - Bmp.Width * S - Bmp.Height * C) / 2;
    end;
    Points[0].X:=Round(OffsetX);
    Points[0].Y:=Round(OffsetY);
    Points[1].X:=Round(OffsetX + Bmp.Width * C);
    Points[1].Y:=Round(OffsetY + Bmp.Width * S);
    Points[2].X:=Round(OffsetX - Bmp.Height * S);
    Points[2].Y:=Round(OffsetY + Bmp.Height * C);
    PlgBlt(Tmp.Canvas.Handle, Points, Bmp.Canvas.Handle, 0, 0, Bmp.Width, Bmp.Height, 0, 0, 0);
    Bmp.Assign(Tmp);
  finally
    Tmp.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  EXIF:=TEXIF.Create;
end;

procedure TForm1.cmOpen(Sender: TObject);
begin
  if opd.Execute then begin
    EXIF.LoadFromFile(opd.Filename);
    Edit1.Text:=opd.Filename;
    tcChange(nil);
  end;
end;

procedure TForm1.ListDblClick(Sender: TObject);
var
  s: string;
begin
  try
    s:=TListView(Sender).Selected.SubItems[1];
    InputQuery(TListView(Sender).Selected.SubItems[0],TListView(Sender).Selected.Caption,s);
  except
  end;
end;

procedure TForm1.tcChange(Sender: TObject);
begin
  List.Hide;
  ImagePanel.Hide;
  Memo.Hide;
  case tc.TabIndex of
    0: DisplayIFD0;
    1: DisplayEXIFSubIFD;
    2: DisplayEXIFInteropIFD;
    3: DisplayMakerNote;
    4: DisplayIFD1;
    5: DisplayEXIFGPSIFD;
    6: DisplayImage;
    7: DisplayThumbnail;
    8: DisplayComment;
  end;
end;

procedure TForm1.DisplayComment;
begin
  Memo.Show;
  Memo.Lines.Text:=EXIF.Comment;
end;

procedure TForm1.DisplayEXIFGPSIFD;
var
  i: Integer;
begin
  with EXIF do begin
    List.Items.BeginUpdate;
    try
      List.Items.Clear;
      for i:=0 to GPSCount-1 do
        with List.Items.Add do begin
          Caption:=Format('0x%x',[GPS_Index[i].Header.Tag]);
          SubItems.Add(GPS_Index[i].Header.Name);
          SubItems.Add(TranslateGPS(GPS_Index[i]));
          ImageIndex:=-1;
        end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  List.Show;
end;

procedure TForm1.DisplayEXIFInteropIFD;
var
  i: Integer;
begin
  with EXIF do begin
    List.Items.BeginUpdate;
    try
      List.Items.Clear;
      for i:=0 to INTEROPCount-1 do
        with List.Items.Add do begin
          Caption:=Format('0x%x',[INTEROP_Index[i].Header.Tag]);
          SubItems.Add(INTEROP_Index[i].Header.Name);
          SubItems.Add(TranslateINTEROP(INTEROP_Index[i]));
          ImageIndex:=-1;
        end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  List.Show;
end;

procedure TForm1.DisplayEXIFSubIFD;
var
  i: Integer;
begin
  with EXIF do begin
    List.Items.BeginUpdate;
    try
      List.Items.Clear;
      for i:=0 to EXIFCount-1 do
        with List.Items.Add do begin
          Caption:=Format('0x%x',[EXIF_Index[i].Header.Tag]);
          SubItems.Add(EXIF_Index[i].Header.Name);
          SubItems.Add(TranslateEXIF(EXIF_Index[i]));
          ImageIndex:=-1;
        end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  List.Show;
end;

procedure TForm1.DisplayIFD0;
var
  i: Integer;
begin
  with EXIF do begin
    List.Items.BeginUpdate;
    try
      List.Items.Clear;
      for i:=0 to TIFFCount-1 do
        with List.Items.Add do begin
          Caption:=Format('0x%x',[TIFF_Index[i].Header.Tag]);
          SubItems.Add(TIFF_Index[i].Header.Name);
          SubItems.Add(TranslateTIFF(TIFF_Index[i]));
          ImageIndex:=-1;
        end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  List.Show;
end;

procedure TForm1.DisplayMakerNote;
var
  i: Integer;
begin
  with EXIF do begin
    List.Items.BeginUpdate;
    try
      List.Items.Clear;
      for i:=0 to MakerNoteCount-1 do
        with List.Items.Add do begin
          Caption:=Format('0x%x',[MakerNote_Index[i].Header.Tag]);
          SubItems.Add(MakerNote_Index[i].Header.Name);
          SubItems.Add(TranslateMakerNote(MakerNoteFormat,MakerNote_Index[i]));
          ImageIndex:=-1;
        end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  List.Show;
end;

procedure TForm1.DisplayIFD1;
var
  i: Integer;
begin
  with EXIF do begin
    List.Items.BeginUpdate;
    try
      List.Items.Clear;
      for i:=0 to EXIF_IFD1Count-1 do
        with List.Items.Add do begin
          Caption:=Format('0x%x',[EXIF_IFD1_Index[i].Header.Tag]);
          SubItems.Add(EXIF_IFD1_Index[i].Header.Name);
          SubItems.Add(TranslateTIFF(EXIF_IFD1_Index[i]));
          ImageIndex:=-1;
        end;
    finally
      List.Items.EndUpdate;
    end;
  end;
  List.Show;
end;

procedure TForm1.DisplayImage;
var
  jpg: TJPEGImage;
  a: Real;
begin
  if not FileExists(EXIF.Filename) then
    Exit;
  jpg:=TJPEGImage.Create;
  try
    jpg.LoadFromFile(EXIF.FileName);
    Image.Picture.Bitmap.Assign(jpg);
    a:=0;
    case Trunc(EXIF.TIFF_Tag[$0112].NumericValue[0]) of
      3,4: a:=Pi;
      5,6: a:=Pi/2;
      7,8: a:=-Pi/2;
    end;
    if a<>0 then
      RotateBitmap(Image.Picture.Bitmap,a,True,clBtnFace);
  finally
    jpg.Free;
  end;
  ImagePanel.Show;
  Label1.Caption:=Format('%d x %d',[Image.Picture.Width,Image.Picture.Height]);
end;

procedure TForm1.DisplayThumbnail;
begin
  if Assigned(EXIF.Thumbnail) and (EXIF.Thumbnail.Height>0) then begin
    Image.Stretch:=False;
    Image.Picture.Assign(EXIF.Thumbnail);
    ImagePanel.Show;
    Label1.Caption:=Format('%d x %d',[Image.Picture.Width,Image.Picture.Height]);
  end;
end;

end.
